/*
Copyright (c) 2008-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description:Defines RSecureSession and RSecureSubSession classes. 
*/



/**
 @file
*/
#ifndef __SECURECLIENT_H__
#define __SECURECLIENT_H__

#include <e32base.h>

/**
The client-side handle to a session with the server.
The class forms a layer over the Symbian provided RSessionBase class.
The main example code calls member functions in the subsession class to make requests to the server. The subsession forwards the requests to the server through the session.
A request is identified by a number. This is one of the TSecureServerRequest enum values defined in secureclientandserver.h, The session passes the number and any arguments to the server in a TIpcArgs object.
@see RSecureSubSession
*/
class RSecureSession : public RSessionBase
	{
public:
	RSecureSession();
	TInt Connect();
	TVersion Version() const;
	TInt ResourceCountL();
	void Close();
private:
	RThread iServerThread;
	};

/**
The client-side handle to a subsession with the server.
The class forms a layer over the Symbian provided RSubSessionBase class.
The main example code calls member functions in this class to make requests to the server. The class forwards the requests to the server through the session.
@see RSecureSession
*/
class RSecureSubSession : public RSubSessionBase
	{
public:
	TInt Open(RSecureSession& aServer);
	TInt SetFromString(const TDesC& aString);
	void Close();
	void Increase();
	void Decrease();
	void IncreaseByL(TInt anInt);
	void DecreaseByL(TInt anInt);
	void Reset();
	TInt CounterValueL();
	void VerifyTransaction();
	void SaveCounterL();
	void SetCounterFromFileL();
	};

#endif

